// Tests if logging into all apps works using SSO
// Unfortunately we still can't run this test in CI because we haven't found
// a way to use SSO with LE staging certs.
// See https://open.greenhost.net/openappstack/single-sign-on/-/issues/62

const { openBrowser, goto, textBox, into, write, click, toRightOf, below, link, press, image, waitFor, closeBrowser, screenshot } = require('taiko');
const assert = require('assert');

(async () => {
  try {
    const taikoTests = process.env.TAIKO_TESTS || 'all'
    const username = process.env.SSO_USERNAME
    const pw = process.env.SSO_USER_PW
    const domain = process.env.DOMAIN
    const adminpanelUrl = 'https://admin.' + domain
    const grafanaUrl = 'https://grafana.' + domain
    const globalTimeout = 60000

    // https://docs.taiko.dev/api/setconfig/
    // setConfig( { observeTime: 1000});
    setConfig( { observeTime: 0, navigationTimeout: globalTimeout });

    console.log('Executing these tests: ' + taikoTests)
    console.log('• Login to admin panel')
    await openBrowser()
    // await screencast.startScreencast('output.gif')

    await goto(adminpanelUrl)
    await click('Login')
    await click('Login with OAS')
    await write(username, into(textBox('Username')))
    await write(pw, into(textBox('Password')))
    await click('Remember me')
    await click('Sign in')

    // Nextcloud
    if (taikoTests.includes('nextcloud') || taikoTests === 'all') {
      console.log('• Nextcloud and Onlyoffice')
      await click(link(below('nextcloud')))
      await click('Log in with OpenAppStack')
      await click('Continue with ' + username)

      // Close potential nextcloud first run wizard modal
      // https://github.com/nextcloud/firstrunwizard/issues/488
      // Unfortunately, we need to sleep a while since I haven't found a
      // good way that closes the modal *if* it pops up, since these
      // tests should also work on subsequent logins.
      await waitFor(5000)
      await press('Escape')

      await assert.ok(await text('Add notes, lists or links …').exists());
    }

    // Rocketchat
    if (taikoTests.includes('rocketchat') || taikoTests === 'all') {
      // Navigate to rocketchat
      // Breaks because of 2fa challenge which is not recieved
      // by email
      // https://open.greenhost.net/openappstack/openappstack/-/issues/819
      // await click(link(below('rocketchat')));
      // await click('Login via OpenAppStack');
      // await click('Continue with ' + username)
    }

    // Wordpress
    if (taikoTests.includes('wordpress') || taikoTests === 'all') {
      console.log('• Wordpress')
      await goto(adminpanelUrl)
      await click(link(below('wordpress')))
      await click('Log in')
      await click('Login with OpenID Connect')
      await click('Continue with ' + username)
      await assert.ok(await link('Dashboard').exists())
    }

    // Grafana
    if (taikoTests.includes('grafana') || taikoTests === 'all') {
      console.log('• Grafana')
      await goto(adminpanelUrl)
      await click(link(below('grafana')))
      await click('Sign in with OpenAppStack')
      await click('Continue with ' + username)
      await assert.ok(await text('Welcome to Grafana').exists());
    }
  } catch (error) {
    await screenshot()
    // await screencast.stopScreencast()
    console.error(error)
    process.exitCode = 1
  } finally {
    // await screencast.stopScreencast()
    await closeBrowser()
  }
})()
